function c = gt(a,b)
% Greater than.
%     A > B does element by element comparisons between A and B
%     and returns a matrix of the same size with elements set to logical 1
%     where the relation is true and elements set to logical 0 where it is
%     not.  A and B must have the same dimensions unless one is a
%     scalar. A scalar can be compared with any size array.
%  
%     C = GT(A,B) is called for the syntax 'A > B' when A or B is an
%     object.

% 4/27/2006: PJS  Initial Coding  

% Promote a to polynomial 
a = polynomial(a);
sza = size(a);

% Promote b to polynomial 
b = polynomial(b);
szb = size(b);


if isempty(a) || isempty(b)
        
    if isempty(a) && all(szb==[1 1])
        % empty>=scalar = empty(sza)
        c=polynomial(zeros(sza));
        return;
    elseif isempty(b) && all(sza==[1 1])
        % scalar>=empty = empty(szb)
        c=polynomial(zeros(szb));
        return;
    elseif all(sza==szb)
        c=polynomial(zeros(sza));
        return;
    else
        error('Matrix dimensions must agree.');
    end   
    
elseif all(sza==szb)
  
    % Matrix >= Matrix
    L.type = '()';
    c = zeros(sza);
    for i1=1:sza(1)
        for i2=1:sza(2)          
            % tmp orders all monominals from a & b 
            L.subs = {[i1] [i2]};
            aij = combine(subsref(a,L));
            bij = combine(subsref(b,L));
            tmp = [aij; bij];
            nterms = get(tmp,'nterms');
            %tcoef = reshape(tmp.coefficient,nterms,2);          
            tcoef = tmp.coefficient;

            % find first term that is not common among aij & bij
            idx = min(find( tcoef(:,1)~=tcoef(:,2) ));
            if ~isempty(idx)
                if (tcoef(idx,1)>tcoef(idx,2)) %|| (tcoef(idx,2)==0)
                    c(i1,i2) = 1;
                end
            end
        end
    end
    c = logical(c);
      
elseif all(sza==[1 1]) | all(szb==[1 1])  
    % Scalar >= Matrix / Matrix >= Scalar

    % Make first term be the scalar.  
    if ~all(sza==[1 1])
        temp = a;
        a = b;

        b = temp;
        szb = sza;
    end

    % Turn into matrix>=matrix
    a.coefficient = repmat(a.coefficient,[1 szb(1)*szb(2)]);
    a.matdim = szb;
    c = ge(a,b);
  
else  
    error('Matrix dimensions must agree');
end










