function varargout = mpvar(var,n,m,opt);
% function P = mpvar(cstr,N,M,opt);
%
% DESCRIPTION 
%   Create a polynomial matrix or vector variable
%   
% INPUTS 
%   cstr: Character string to be used in creating the coefficient vector.
%   N,M: row and column dimensions of polynomial matrix.
%   opt: If N==M, then set opt = 's' to generate a symmetric 
%        matrix variable.
%
% OUTPUTS  
%   P: polynomial matrix 
%  
% SYNTAX 
%   P = mpvar(cstr,N,M)
%       Creates an NxM polynomial matrix with entries
%       cstr_i_j. 
%   mpvar(cstr,N,M)
%      Equivlavent to calling eval([cstr '=mpvar(cstr,N,M);']).
%
% EXAMPLE
%   P = mpvar('p',2,3)

% 11/10/2002 PP    Initial Coding
% 11/10/2002 PJS   Minor modifications for speed and allow 
%                  calling sequence with no outputs
  
  
% Argument checking
error(nargchk(3,4,nargin))
if nargin==3
    opt = [];
end
error(nargchk(0,1,nargout))

if n~=m && strcmp(opt,'s')
    error('Symmetric option only valid for square matrices');
end
    
% Form Coefficients
nt = n*m;
coefficient = speye(nt);

% The degrees
degmat = speye(nt);

% Create the names
stridx = int2str( (1:max([n m]))' );
stridx = strjust( stridx ,'left');
stridx = cellstr(stridx);
varname = cell(nt,1);
for i1=1:n ;
    tempvar = [var '_' stridx{i1} '_'];
    for i2 =1:m;
        varname{i1+(i2-1)*n} = [tempvar stridx{i2}];    
    end
end

% Create matrix dimension
matdim = [n m];
chkval = 0; % skip validity check
P = polynomial(coefficient,degmat,varname,matdim,chkval);
if strcmp(opt,'s')            
    % Single indices into lower/upper parts of a symmetric matrix
    % sorted so that single index for (i,j) is aligned with (j,i)
    M = reshape(1:n^2,[n n]);    
    Ml = tril(M,-1);
    Ms = Ml+Ml';
    [junk,idx]=sort(Ms(:)); 
    idx=reshape(idx(n+1:end),[2,n*(n-1)/2]);
    lidx = idx(1,:);
    uidx = idx(2,:);
    
    % Create symmetric matrix of pvars
    Psym = P;
    Psym(lidx) = Psym(uidx);
    P = Psym;    
end
    
if nargout==0
    assignin('caller', var, P);
else
    varargout{1} = P;
end
