%% Robust ROA calculations

%%
% dynamics:
%%
% x1dot = x2;
%%
% x2dot = -x2-2*x1+2*x1^3 + delta*(-x1+x1^3);
%%
% with delta \in [-1,1]

%% 
% This example was also used in Topcu and Packard, IEEE TAC, 2009 (in the
% special issue on positive polynomials in controls (example 1 in the
% paper)




% Form the vector field
pvar x1 x2;
x = [x1;x2];
x1dot = x2;
x2dot = -x2-2*x1+2*x1^3;
%%
% Nominal system
f = [x1dot; x2dot];
%%
% Introduce an uncertain parameter 
pvar d1
%%
% Specify its range
ini_cell = [-1 1];
%%
% Form the uncertain vector field
f = f + d1*[0; -x1+x1^3];


% Get the vertex system
[roaconstr,opt,sys] = GetRoaOpts(f, x);
[fNOM,fVER] = getf(sys,ini_cell);

% Generate the options, etc.
zV = monomials(x,2:4);
Bis.flag = 0;
Bis.r1deg = 4;


[roaconstr,opt,sys] = GetRoaOpts(fVER, x, zV, [], Bis);
sys.fWithDel = [];

opt.sim.NumConvTraj = 40;
opt.display.roaest = 1;

%%
% Run the computations
outputs = wrapper(sys,[],roaconstr,opt);
%%
% Extract the solution
[V,beta,gamma,p,multip,betaUpper] = extractSol(outputs);

%%
beta
%% 
% Upper bound on beta
betaUpper


%%
% Plot the results
[Cp4,hp4] = pcontour(p,beta,[-2 2 -2 2],'k'); hold on;
set(hp4,'linewidth',2);
[CV4,hV4] = pcontour(V,gamma,[-2 2 -2 2],'b');
set(hV4,'linewidth',2);
set(gca,'xlim',[-1.5 1.5],'ylim',[-1.5 1.5]);

traj = outputs.RoaEstInfo.info.SimLFG.sim.Trajectories(1).unstab(end).state;
pval = peval(traj,p.coef,p.deg);
[aux,ind] = min(pval);
plot(traj(1,ind),traj(2,ind),'r*','markersize',8);
grid on;





