%
% This File performs the approximation for some of the Aerodynamic
% Coefficients from the 9 State/ 6DoF model described in 'F18Analysis \
% 1_F18FullModel' folder. 
% References to the Aerodynamic Data have been provided below. 


% Unit Conversion : Degree <--> Radian
d2r = pi/180;
r2d = 1/d2r;


%========================================================================
% Rolling Moment Coefficient due to Sideslip Approximation

% Rolling moment Due to Sideslip Angle
% Reference: 
%           Marcello R. Napolitano and Joelle M. Spagnuolo,
%           Determination of the stability and control derivatives of the 
%           NASA F/A-18 HARV using flight data, NASA CR-194838, 1993.

% Flight Test Data
Cl_b  =     [-0.0012  -0.0025   -0.0031   -0.0033  -0.0028   -0.0025  -0.0020  -0.0016  -0.0012]*r2d;   % Unit was 1/deg
alpha_roll = [  5        10       15        20        25       30        35       40       45]*d2r;

%----------------------------
% Fit of Nth order.  F18FullModel uses 3rd order fit. 
N = 2;   % 2nd Order Approximation 

figure
plot(alpha_roll*r2d,Cl_b*d2r,'rx')
hold on
p0 = polyfit(alpha_roll,Cl_b,N);
f = polyval(p0,alpha_roll);
plot(alpha_roll*r2d,f*d2r,'--')
legend('flight test data','Estimated Fit')
grid on 
xlabel('alpha (deg)')
ylabel('Cl_b ( 1/deg)')


%========================================================================
% Sideforce Coefficient due to Sideslip Approximation

% Reference: 
%           Marcello R. Napolitano and Joelle M. Spagnuolo,
%           Determination of the stability and control derivatives of the 
%           NASA F/A-18 HARV using flight data, NASA CR-194838, 1993.


% SideForce Data due to SideSlip
Cy_b        =  [-0.013  -0.015   -0.014   -0.013   -0.011  -0.0085   -0.007   -0.0065  -0.006]*r2d ; % Unit was 1/deg
alpha_Y     =  [ 5         10      15       20       25      30        35       40        45 ]*d2r; 

figure
plot(alpha_Y*r2d,Cy_b*d2r,'rx')
hold on
p6 = polyfit(alpha_Y,Cy_b,N);
f = polyval(p6,alpha_Y);
plot(alpha_Y*r2d,f*d2r,'--')
legend('flight test data','Estimated Fit')
grid on 
xlabel('alpha (deg)')
ylabel('CY_b ( 1/deg)')