function [BetaUB, V, info] = SimLFG(fList, XList, roaconstr, opt)
% function [BetaUB, V, info] = simLFG(fList, XList, roaconstr, opt)
%
% DESCRIPTION 
%   simulates polynomial dynamic system, generates LP constraints to be
%   satisfied by a Lyapunov function, and solves for a candidate 
%   Lyapunov function.
%   
% INPUTS 
%   fList: 
%   XList: 
%   roaconstr:
%   opt: 
%
% OUTPUTS 
%   BetaUB: 
%   V:
%   info: 
%
% SYNTAX 
%   [BetaUB, V, info] = simLFG(fList, XList, roaconstr, opt)
%
% REFERENCE
%   Topcu, Packard, and Seiler,  "Local Stability Analysis Using 
%   simulations and Sum-of-Squares Programming," to appear in Automatica.
%
% 1/23/08  UT/PJS     Initial code re-structuring based on Ufuk's code
%

% if 0
%-------------------------------------------------
% Get Options
%-------------------------------------------------
% display('In SimLFG');
d = opt.display.roaest;
dsim = opt.sim.dispEveryNth;

% roaconstr. p = 0.40146*XList(1)^2 - 0.14771*XList(1)*XList(2) + 0.35708*XList(2)^2;
p = roaconstr.p;

% ['SimLFG1']
%--------------------------------------------
% To see the effect of having the Lyap function from quadratic analysis as
% % shape factor
% p = XList'*opt.pM*XList;
% p = XList'*[ 1.0908   -0.2195
%    -0.2195    1.2205]*XList;
% %---------------------------------------------
% ['SimLFG2']
zV = roaconstr.zV;

BetaInit = opt.sim.BetaInit;
BetaMax = opt.sim.BetaMax;
if BetaMax < BetaInit
    BetaInit = BetaMax;
end
BetaShrink = opt.sim.BetaShrink;
BetaGrow = opt.sim.BetaGrow;
NumConvTraj = opt.sim.NumConvTraj;
tfinal = opt.sim.tfinal;
event_params = opt.sim.event_params;


%-------------------------------------------------
% Run simulations
%-------------------------------------------------
if d
    fprintf('------------------Beginning simulations\n');
end
% tsim = tic;
tic;


BetaUB = inf;
Beta = BetaInit;
szF = size(fList);
Nsys = szF(2);
Xorigin = zeros(szF(1),1);





simStatus = cell(Nsys,1);
for i1 = 1:Nsys
    simData(i1).unstab(1).time = [];
    simData(i1).unstab(1).state = []; 
    simData(i1).unstab(1).idxsmallestp = []; 
    
    scnt = 0; 
    ucnt = 0;
        
    simStatus{i1} = zeros(2*NumConvTraj,4);    
    simStatus{i1}(1,:) = [scnt ucnt Beta BetaUB];
    while scnt < NumConvTraj
        % Sample {x: p(x)<=Beta} for an initial condition
        % XXX--This assumes the origin is in the interior of the set
        % XXX PJS 5/20/2009 Updated to psample
        [Xin,X0]=psample(p-Beta,XList,Xorigin,1);
        %[Xin,X0] = polyset_sample(p,Beta,1,Xorigin);
        
        % simulate Polynomial Dynamical System
        % XXX PJS 5/20/2009 Updated to psim
        [xtraj,isconv]=psim(fList(:,i1),XList,X0,tfinal,event_params);
        trajTX.time = xtraj{1,1}';
        trajTX.state = xtraj{1,2}'; 
%         sys.num = fList(:,i1);
%         sys.den = 1;
%         sys.state = XList;
%         [xtraj,isconv]=sim_rsys(sys,X0,tfinal,event_params);
%         trajTX.time = xtraj{1}(1,:);
%         trajTX.state = xtraj{1}(2:end,:);
        
        % Store data and grow or shrink the simulation
        % area depending on the results
        if isconv == 1
            scnt = scnt + 1;
            simData(i1).stab(scnt).time = trajTX.time;
            simData(i1).stab(scnt).state = trajTX.state;            
            if (ucnt == 0) & (Beta<BetaMax) & (Beta<BetaUB)
                Beta = min(BetaGrow*Beta,BetaMax);
            end
        else

            % XXX PJS 5/20/2009 Updated to psim
            [xxxB,isconvB]=psim(fList(:,i1),XList,X0,-opt.sim.tfinalBackForDiv,event_params);
            trajTX.time = [fliplr(xxxB{1,1}') trajTX.time];
            trajTX.state = [fliplr(xxxB{1,2}') trajTX.state];
%             [xxxB,isconvB]=sim_rsys(sys,X0,-opt.sim.tfinalBackForDiv,event_params);
%             trajTX.time = [fliplr(xxxB{1}(1,2:end)) trajTX.time];
%             trajTX.state = [fliplr(xxxB{1}(2:end,2:end)) trajTX.state];
                
            % Each unstable traj provides a hard upper bound on Beta
            [pmin,idxmin] = min(peval(trajTX.state, p.coef,p.deg));
            BetaUB = min(BetaUB, pmin);

            ucnt = ucnt + 1;
            simData(i1).unstab(ucnt).time = trajTX.time;
            simData(i1).unstab(ucnt).state = trajTX.state; 
            simData(i1).unstab(ucnt).idxsmallestp = idxmin;
            Beta = min(pmin, BetaShrink*Beta);
        end    
        
        
        idx = scnt+ucnt+1;
        if idx > size(simStatus{i1},1)
            simStatus{i1} = [simStatus{i1}; zeros(NumConvTraj,4)]
        end
        simStatus{i1}(idx,:) = [scnt ucnt Beta BetaUB]; 
        if d &  (~isconv | floor(scnt/dsim)==ceil(scnt/dsim))
            fprintf('System %d: ',i1);
            fprintf('Num Stable = %d \t',scnt);
            fprintf('Num Unstable = %d \t',ucnt);
            fprintf('Beta for Sims = %4.3f \t',Beta);
            fprintf('Beta UB = %4.3f \n',BetaUB);
        end                        
    end
    if ucnt==1
        simData(i1).unstab = [];
    end
    simStatus{i1} = simStatus{i1}(1:idx,:);
end

% tsim = toc(tsim);
tsim = toc;

if d
    fprintf('------------------End of simulations\n');
end

% for i12 = 1:length(simData.unstab)
%     simData.unstab(i12).time = single(simData.unstab(i12).time);
%     simData.unstab(i12).state = single(simData.unstab(i12).state);
% end
% for i12 = 1:length(simData.stab)
%     simData.stab(i12).time = single(simData.stab(i12).time);
%     simData.stab(i12).state = single(simData.stab(i12).state);
% end


% Store information about simulations
info.sim.CPUTime = tsim;
info.sim.BetaUB = BetaUB;
info.sim.Trajectories = simData;
info.sim.Status = simStatus;
info.sim.Comment = {'Columns of sim.Status{i} are: '; ...
    '1) # of stable trajectories'; ...
    '2) # of unstable trajectories'; ...
    '3) Beta for simulation ICs'; ...
    '4) Beta Upper Bound'};

% end

% load SimDataDouble
% for i23 = 1:2500
%     info.sim.Trajectories.stab(i23).time = single(info.sim.Trajectories.stab(i23).time);
%     info.sim.Trajectories.stab(i23).state = single(info.sim.Trajectories.stab(i23).state);
% end
%     

%-------------------------------------------------
% Find Candidate Lyapunov Function
%-------------------------------------------------

if d
    fprintf('------------------Begin search for feasible V\n');
end

%tVfeas = tic;
tic
c = [];
go = 1;
cnt = 1;
VFeasStatus = [cnt Beta];    

Aapp = []; bapp = []; b1app = [];

V = [];


while go & cnt <= opt.lp.timestosolvelp & Beta>opt.lp.MinBetaToTryLP

    % Display Information
    if d
       fprintf('Try = %d \t Beta for Vfeas = %4.3f\n',cnt,Beta);
    end
    
    % Convert trajectories to LP

    [A,b,b1] = traj2LP(simData, fList, XList, Beta, roaconstr, opt.lp);
    A = [A;Aapp];
    b = [b;bapp];
    b1 = [b1;b1app];
    
    
%     cd /home/a/utopcu/matlab/NA_Code/examples
%     save LPdatafile A b
%     cd /home/a/utopcu/matlab/NA_Code/roacode
    
    % Find a feasible candidate V
    if opt.lp.sampleForPositive
        [c,Vfeasinfo] = findVfeas_OnlyLP(A, b, opt.findVfeas);
    else
        if opt.findVfeas.feasonly
            if opt.findVfeas.useYalmip 
                [c,Vfeasinfo] = findVfeas_Yalmip(fList,A(:,1:end-1),b1, roaconstr, opt.findVfeas);
            else
                [c,Vfeasinfo] = findVfeas_New(fList,A(:,1:end-1),b1, roaconstr, opt.findVfeas);
            end
        else
            if opt.findVfeas.useYalmip
                [c,Vfeasinfo,ep] = findVfeas_Yalmip(fList,A,b, roaconstr, opt.findVfeas);
            else
                [c,Vfeasinfo,ep] = findVfeas_New(fList,A,b, roaconstr, opt.findVfeas);
            end
        end
    end
   
    if isempty(c) 
        BetaUB = Beta;
        Beta =BetaShrink*Beta;
              
    else
        go = 0;
    end
    
    VFeasStatus = [VFeasStatus; cnt Beta];        
    cnt = cnt+1;
end


if ~isempty(c)
    V = c'*zV;   
    if d
       fprintf('------------------Found feasible V\n');
    end
else
    V = [];
    if d
        fprintf('------------------Max number of trials to solve the convex prog. has been reached\n');
        fprintf('------------------Infeasible problem beta upper bound is set to 0\n');
    end
    BetaUB = 0;
end
    

% tVfeas = toc(tVfeas);
tVfeas = toc;


% Store information about search for feasible V
info.Vfeas.CPUTime = tVfeas;
info.Vfeas.V = V;
info.Vfeas.c = c;
if ~opt.findVfeas.feasonly info.Vfeas.ep = ep; end
info.Vfeas.Status = VFeasStatus;
info.Vfeas.Comment = {'Columns of Vfeas.Status are: '; ...
    '1) Iteration Number'; ...
    '2) Beta Attempted'};

info.LP.A = A;
info.LP.b = b;
info.LP.b1 = b1;



        
