function info = getgamma(V, fList, XList, roaconstr, opt)
% function info = getgamma(V, fList, XList, roaconstr, opt)
%
% DESCRIPTION 
%
%   
% INPUTS 
%   V: 
%   fList: 
%   XList: 
%   opt: 
%
% OUTPUTS 
%   info: structure with fields Gamma, S2, S3.
%
% SYNTAX 
%   info = getgamma(V, fList, XList, opt)
%
%
% 1/23/08  UT/PJS     Initial code re-structuring based on Ufuk's code
%

% No error checking


% Set Options
d = opt.display;
GammaHigh = opt.maxgamma;
GammaLow = opt.mingamma;
bistol = opt.bistol;
usesosp = opt.usesosp;
sedopts = opt.sedopts;

z2 = roaconstr.z2;
z3 = roaconstr.z3;
L2 = roaconstr.L2;
mu = roaconstr.mu;

szF = size(fList);
Nsys = szF(2);

% Initalize Values for Bisection
LookAt = 1:Nsys;
GammaOut = []; 
S2Out = polynomial([]);  
S3Out = polynomial([]);  

if GammaLow>0
    % Verify GammaLow is a true lower bound on the first pass
    Gamma = GammaLow;
    GammaLow = 0;
else
    Gamma = GammaHigh/2;
end

% Perform Bisection
Vd = jacobian(V,XList)*fList;
while GammaHigh-GammaLow>bistol   
    % Display Bisection Status
    if d 
        fprintf('GammaLow  = %4.3f \t',GammaLow);
        fprintf('GammaTry = %4.3f \t',Gamma);
        fprintf('GammaHigh = %4.3f \n',GammaHigh);
    end                        
        
    go = 1; cnt = 1;
    while go
        index = LookAt(cnt);
        [isfeasp, S2, S3] = LOCALVdotcontainV(V, Gamma, XList, ...
            Vd(:,index) , z2, z3, L2, mu,usesosp,sedopts);
        
        if ~isfeasp
            LookAt(1:cnt-1) = [];
            GammaHigh = Gamma;
            go = 0;
        else
            S2Out(LookAt(cnt)) = S2;
            S3Out(LookAt(cnt)) = S3;
            if cnt==length(LookAt)
                go = 0;
                GammaLow = Gamma;
                GammaOut = Gamma;
            else
                cnt = cnt + 1;
            end
        end
    end    
    
    % Set Gamma level for next feasibility problem
    Gamma = 0.5*(GammaLow+GammaHigh);
end

% Set output values of Gamma, S2, and S3
if isempty(GammaOut) GammaOut = 0; end
info.Gamma = GammaOut;
info.S2 = S2Out;
info.S3 = S3Out;


%-----------------------------------------------------------
% Local Function: 
%   Given fixed Gamma and V.
%   Find S2 and S3 which satisify Equation 8 in the Automatica Paper.
%   ( But modified to ensure that Vdot <= -mu*V )
%-----------------------------------------------------------
function [isfeasp, S2, S3] = LOCALVdotcontainV(V,Gamma, XList, Vd, ...
        z2, z3, L2, mu, usesosp,sedopts)

%usesosp = 1;   

if 1 %usesosp
    % XXX PJS 5/21/2009 Updated old sosp code to new sosopt 
    % (Hard-coded to run sosopt rather than rely on usesosp option)
    S2 = polydecvar('S2',z2,'mat');
    S3 = polydecvar('S3',z3,'mat');
    ineq{1} = S2;
    ineq{2} = S3;
    ineq{3} = -((Gamma-V)*S2 + (Vd+mu*V)*S3 + L2);
    [info,dopt]=sosopt(ineq,XList,[],[],sedopts);
    isfeasp = info.feas;
    if isfeasp
        S2 = subs(S2,dopt(:,1),dopt(:,2));
        S3 = subs(S3,dopt(:,1),dopt(:,2));
    else
        S2 = [];
        S3 = [];
    end
            
else 
    prog = sosprogram(XList);

    [prog,S2] = sossosvar(prog,z2);
    [prog,S3] = sossosvar(prog,z3);

    ineq1 = -((Gamma-V)*S2 + (Vd+mu*V)*S3 + L2);
    prog = sosineq(prog,ineq1);
    [prog,info] = sossolve(prog,[],[],'sedumi');

    isfeasp = double(check_feasp(info,'sedumi')==1);
    S2 = sosgetsol(prog, S2);
    S3 = sosgetsol(prog, S3);
end


