
function [ax1, fig1, H] = plot_level(Vstack,options)
% plotlevel1 plots 1 'One' level set of pointwise max of functions in Vstack{i}
% Vstack{i} contains polynomial object ( with (possibly) more than one polynomials).
% options is truct containing data for creating the contour data and plotting
% options.limits --> limits for creating contour data, if it does not exist
% it is set to default [-3 3 -3 3], for different limits for each element
% of Vstack limits should be cell array. This is true for all field of
% options except limitsPlot which is a row vector. Defaults for options
% fields are assigned in the code (below).

if nargin==1
    options=struct;
end


pvar x1 x2
x=[x1;x2];
for i=1:length(Vstack)
    Vstack{i}.varname={'x1','x2'};
end

if ~isfield(options,'limits')
   limits = {[-3 3 -3 3]}; % limits for creating countour data [x1min x1max x2min x2max]
else
    limits=options.limits;
end
if ~isfield(options,'stepSize')
   stepSize = {[.06 .06]}; % step size for creating contour data [x1Step x2Step]
else
    stepSize=options.stepSize;
end 
if ~isfield(options,'LStyle')
   LStyle = {'k-'}; % color and style of the contour curve
else
    LStyle=options.LStyle;
end 
if ~isfield(options,'LWidth')
   LWidth = {2}; %width of the contour curve
else
    LWidth=options.LWidth;
end 

if ~isfield(options,'limitsPlot')
    limitsPlot = [-3 3 -3 3]*2; % limits for the plot [x1min x1max x2min x2max]
else
    limitsPlot=options.limitsPlot;
end

if length(Vstack)>1
    if length(limits)==1
        for k=2:length(Vstack)
        limits{k}=limits{1};
        end
    end
    if length(stepSize)==1
        for k=2:length(Vstack)
        stepSize{k}=stepSize{1};
        end
    end
    if length(LStyle)==1
        for k=2:length(Vstack)
        LStyle{k}=LStyle{1};
        end
    end
    if length(LWidth)==1
        for k=2:length(Vstack)
        LWidth{k}=LWidth{1};
        end
    end
end
  
fig1 = figure(1)
for k=1:length(Vstack)
    [x1d,x2d]=ndgrid(limits{k}(1):stepSize{k}(1):limits{k}(2),limits{k}(3):stepSize{k}(2):limits{k}(4));
    X=[reshape(x1d,[size(x1d,1)*size(x1d,2) 1]) reshape(x2d,[size(x2d,1)*size(x2d,2) 1])]' ;
    V=Vstack{k};
    Vdata=peval(X,V.coef,V.deg);
    Vd=max(Vdata,[],1);
    Vd=reshape(Vd',size(x1d));
    [C,H(k)]=contour(x1d,x2d,Vd,1*[1 1],LStyle{k});hold on;
    set(H,'linewidth',LWidth{k});
end


ax1 = gca;