function [simdataDecim] = simdecim(simdata, p, Beta, decimN)
% function [simdataDecim] = simdecim(simdata, p, Beta, decimN);
%
% DESCRIPTION 
%   Decimate simulation data
%   
% INPUTS 
%   simdata:
%   p:
%   Beta: 
%   decimN: 
%
% OUTPUTS 
%   A:
%   b:
%
% SYNTAX 
%  [simdataDecim] = simdecim(simdata, p, Beta,decimN)
%
% REFERENCE
%
% 1/23/08  UT/PJS     Initial code re-structuring based on Ufuk's code
%

%-------------------------------------------------
% Decimate Trajectory Data
%-------------------------------------------------
Nsys = length(simdata);
simdataDecim = simdata; 
for i1 = 1:Nsys
    stab = simdataDecim(i1).stab;
    unstab = simdataDecim(i1).unstab;
    Nst = length(stab);
    for i2=1:Nst
        % Remove trajectory points which are outside {x: p(x)<Beta}
        time = stab(i2).time;
        state = stab(i2).state;
        p_x = peval(state,p.coef,p.deg);
        ind = find(p_x<Beta);
        ind = min(ind);

        time = time(ind:end);
        state = state(:,ind:end);

        % Downsample: Use every decimN^th point
        idx = 1:decimN:length(time);
        time = time(idx);
        state = state(:,idx);

        stab(i2).time = time;
        stab(i2).state = state;              
    end
    simdataDecim(i1).stab = stab;
end
