% SOSOPTDEMO4 --- Matrix Copositivity
% Section 3.4 of SOSTOOLS User's Manual
% 


% =============================================
% Define polynomial variables

pvar x1 x2 x3 x4 x5;
x = [x1; x2; x3; x4; x5];

% The matrix under consideration
J = [1 -1  1  1 -1;
    -1  1 -1  1  1;
     1 -1  1 -1  1;
     1  1 -1  1 -1;
    -1  1  1 -1  1];


% =============================================
% Define SOS constraints

% Constraint : r(x)*J(x) - p(x) = 0
J = [x1^2 x2^2 x3^2 x4^2 x5^2]*J*[x1^2; x2^2; x3^2; x4^2; x5^2];
r = x1^2 + x2^2 + x3^2 + x4^2 + x5^2;
sosconstr = r*J;

% =============================================
% And call solver
% Either kernel or image form can be used.
% When form='image', the solver runs into numerical
% problems but the solution (which can be obtained by
% turning of the feasibility check) is still feasible.
% Alternatively, the numerical scaling can be turned on.

opts = [];
%opts.form = 'image'; opts.checkfeas = 'off'
opts.form = 'image'; opts.scaling = 'on';
%opts.form = 'kernel';


[info,dopt,sossol] = sosopt(sosconstr,opts);

% =============================================
% Program is feasible. The matrix J is copositive.
fprintf('\n----------------Results');
if info.feas
    fprintf('\nSOS Optimization is feasible:\n');
else
    fprintf('\nSOS Optimization is not feasible. \n');    
    return
end

% =============================================
% Verify the Gram Marix Decompositions in sossol
w = sossol{2};
Q = sossol{3};
e = sosconstr - w'*Q*w;
fprintf('\nMax magnitude coefficient of p-w''*Q*w is:')
disp(full(max(abs(e.coefficient))))
fprintf('Minimum eigenvalue of Q is:')
disp(min(eig(Q)));





