% SOSOPTDEMO5 --- Upper bound for the structured singular value mu
% Section 3.5 of SOSTOOLS User's Manual
% 

% clear;
i=sqrt(-1);

% =============================================
% Define polynomial variables
pvar x1 x2 x3 x4 x5 x6 x7 x8;
x = [x1; x2; x3; x4; x5; x6; x7; x8];

% The matrix under consideration
alpha = 3 + sqrt(3);
beta = sqrt(3) - 1;
a = sqrt(2/alpha);
b = 1/sqrt(alpha);
c = b;
d = -sqrt(beta/alpha);
f = (1 + i)*sqrt(1/(alpha*beta));
U = [a 0; b b; c i*c; d f];
V = [0 a; b -b; c -i*c; -i*f -d];
M = U*V';

% Constructing A(x)'s
A = [];
gam = 0.8724;
Z = monomials(x,1);
for i = 1:4
    H = M(i,:)'*M(i,:) - (gam^2)*sparse(i,i,1,4,4,1);
    H = [real(H) -imag(H); imag(H) real(H)];
    A{i} = (Z.')*H*Z;
end;

% =============================================
% Define SOS decision variables
sosconstr = [];

% -- Q(x)'s -- : sums of squares
% Monomial vector: [x1; ... x8]
lz = length(Z);
Q = [];
for i = 1:4
    Q{i} = polydecvar(['M' int2str(i)],Z,'mat');
    sosconstr{i,1} = Q{i};
end;

% % -- r's -- : constant sum of squares
Z = monomials(x,0);
r = cell(4,4);
for i = 1:4
    for j = (i+1):4
        pvar(['r_' int2str(i) '_' int2str(j)]);
        r{i,j} = eval(['r_' int2str(i) '_' int2str(j)]);
        sosconstr = [sosconstr; r(i,j)];
    end;
end;

% =============================================
% Next, define SOS constraints

% Constraint : -sum(Qi(x)*Ai(x)) - sum(rij*Ai(x)*Aj(x)) + I(x) >= 0
expr = 0;
% Adding term
for i = 1:4
    expr = expr - A{i}*Q{i};
end;
for i = 1:4
    for j = (i+1):4
        expr = expr - A{i}*A{j}*r{i,j};
    end;
end;
% Constant term: I(x) = -(x1^4 + ... + x8^4)
I = -(x1^4+x2^4+x3^4+x4^4+x5^4+x6^4+x7^4+x8^4);
expr = expr + I;

sosconstr{end+1} = expr;

% =============================================
% And call solver
% Either kernel or image form can be used.
opts = [];
opts.form = 'image';
%opts.form = 'kernel';
[info,dopt,sossol] = sosopt(sosconstr,x,opts);

% =============================================
% Program is feasible, thus 0.8724 is an upper bound for mu.
fprintf('\n----------------Results');
if info.feas
    fprintf('\nSOS Optimization is feasible:\n');
else
    fprintf('\nSOS Optimization is not feasible. \n');    
    return
end

% =============================================
% Verify the Gram Marix Decompositions of sosconstr{end}
p = subs(sosconstr{end},dopt);
w = sossol{end,2};
Q = sossol{end,3};
e = p - w'*Q*w;
fprintf('\nMax magnitude coefficient of p-w''*Q*w is:')
disp(full(max(abs(e.coefficient))))
fprintf('Minimum eigenvalue of Q is:')
disp(min(eig(Q)));

